/* (C) 2022 YiRing, Inc. */
package com.yiring.common.domain;

import com.vladmihalcea.hibernate.type.json.JsonBinaryType;
import com.vladmihalcea.hibernate.type.json.JsonType;
import java.time.LocalDateTime;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.*;
import org.hibernate.snowflake.SnowflakeId;

/**
 * 基础表抽象类
 *
 * @author Jim
 * @version 0.1
 * 2022/4/17 12:11
 */

@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@SuperBuilder(toBuilder = true)
@TypeDefs(
    value = {
        @TypeDef(name = "json", typeClass = JsonType.class), @TypeDef(name = "jsonb", typeClass = JsonBinaryType.class),
    }
)
@MappedSuperclass
public abstract class BasicEntity {

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.STRING)
    String id;

    @Comment("创建时间")
    @Column(nullable = false)
    @CreationTimestamp
    LocalDateTime createTime;

    @Comment("最后修改时间")
    @Column(nullable = false)
    @UpdateTimestamp
    LocalDateTime updateTime;

    @Comment("删除时间")
    LocalDateTime deleteTime;

    public interface Where {
        String EXIST = " delete_time is null ";

        String DELETE_SET = " set delete_time = now() where id = ? ";
    }
}
